(*
   PROJECT          : T-Year initial reserve
   FILE-SPEC.       : initres.sp
   AUTHOR           : R.-D. Reiss, M. Thomas
   CREATION DATE    : May-21-1996
   LAST UPDATE      : Dec-18-2000
   VERSION          : 1.0
   FUNCTIONAL DESCR.: Bootstrap of initial reserve for fire claim data
		      (see Demo 12.4)
   OUTPUT	    : Data set initres.dat with simulated values
   SPECIAL FEATURES : None
*)

program BootRes;

const T = 10.0;

function InitialReserve (gamma, mu, sigma, lambda: real): real;

    const   rho     = 0.1;
            beta    = 1.0;

            sstart  = 230.0;
            q       = 0.05;
    
            SimSize = 4000;
            epsilon = 1e-3;
    
            MaxStep = 10;


    var s0, s1, s2, q0, q1, q2: real;
        Steps: integer;

    function RuinProbability (u: real; SimSize: integer): real;
        var i, ruin: integer;
        begin
            ruin := 0;
            for i := 1 to SimSize do begin
                if SimulateRuinTime (u, rho, beta, lambda, T, gamma, mu, sigma) < T then
                    ruin := succ (ruin);
            end;
            return ruin / SimSize
        end;

    begin
        s0 := sstart;
        q0 := RuinProbability (s0, 4000);
        if q0 > q then
            repeat
                q1 := q0;
                s1 := s0;
                s0 := 2.0 * s0;
                q0 := RuinProbability (s0, 4000);
            until q0 <= q
        else if q0 < q then begin
            s1 := s0;
            q1 := q0;
            repeat
                q0 := q1;
                s0 := s1;
                s1 := s1 / 2.0;
                q1 := RuinProbability (s1, 4000);
            until q1 > q
        end;
        steps := 0;
        repeat
            s2 := ((q1 - q) * s0 + (q - q0) * s1) / (q1 - q0);
            q2 := RuinProbability (s2, SimSize);
            if q2 > q then begin
                q1 := q2;
                s1 := s2 end
            else begin
                q0 := q2;
                s0 := s2
            end;
            steps := succ (steps);
        until (abs (s1 - s2) < epsilon) or (steps = MaxStep);
        if steps = MaxStep then return -1.0;
        return s2
    end;

procedure BootstrapParameter (gamma, mu, sigma, lambda: real;
                              var gammas, mus, sigmas, lambdas: real);
    var i, j: integer;
        sum: real;
        z: vector of real;
    begin
        repeat
            j := 0;
            sum := 0.0;
            while sum < T do begin
                j := j + 1;
                sum := sum + ExponentialData / lambda
            end;
            lambdas := (j - 1) / T;
	    z := mu + sigma * GPData (gamma, j - 1);
            MLEGP (z, j - 1, gammas, mus, sigmas, i)
        until i = 0;
    end;

procedure Simulate (n: integer; lambda, gamma, sigma: real; fn: string);
    const mu = 0.0;
    var z: vector of real;
        i: integer;
        gammas, mus, sigmas, lambdas: real;
    begin
        for i := 1 to n do begin
            BootstrapParameter (gamma, mu, sigma, lambda, gammas, mus, sigmas, lambdas);
            z := combine (z, InitialReserve (gammas, mus, sigmas, lambdas));
            writeln (i, ' of ', n, ' done');
        end;
        createunivariate (z, fn, '')
    end;

begin
    writeln ('Claim size distribution: GP (0.254, 0, 11.948)');
    writeln ('Intensity lambda:        1.7');
    writeln ('Time horizon:            10.0');
    writeln ('Ruin probability:        0.05');
    writeln ('Safety exponent beta:    1.0');
    writeln ('Safety loading rho:      0.1');
    writeln ('Bootstrapping initial reserve - Please wait');
    writeln ('*************************************************');
    Simulate (50, 1.7, 0.254, 11.948, 'initres.dat');
    writeln ('*************************************************');
    writeln ('Done')
end.

